# coding: utf-8

import csv
import codecs
import cStringIO
from functools import partial

from django.utils.encoding import smart_str
from django.conf import settings

import ylock
from ylock.decorators import locked


lock_manager = ylock.create_manager(**settings.YLOCK)
lock = partial(locked, manager=lock_manager)


def replace_form_error_messages(form_class, error_messages):
    for field in error_messages:
        for message in error_messages[field]:
            form_class.base_fields[field].error_messages[message] = error_messages[field][message]


# Copypaste from Python docs
# http://docs.python.org/library/csv.html#examples
class UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """
    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")


class UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, "utf-8") for s in row]

    def __iter__(self):
        return self


class UnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        # Redirect output to a queue
        self.queue = cStringIO.StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        self.writer.writerow([s.encode("utf-8") for s in row])
        # Fetch UTF-8 output from the queue ...
        data = self.queue.getvalue()
        data = data.decode("utf-8")
        # ... and reencode it into the target encoding
        data = self.encoder.encode(data)
        # write to the target stream
        self.stream.write(data)
        # empty queue
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)


def lego_to_html(lego):
    from django_xscript.middleware import do_process_html
    lego = '<stdin>' + lego + '</stdin>'
    stdout = do_process_html(None, smart_str(lego, 'UTF-8'))
    return stdout[stdout.find('<stdin>') + 7:stdout.find('</stdin>')]
