from wiki.acl.check_access import assert_has_access
from wiki.acl.consts import Action

from wiki.api_v2.di import di, legacy_org_ctx, log_slug
from wiki.api_v2.exceptions import IsStubPage
from wiki.api_v2.public.pages.schemas import PageSchema, AclSchema
from wiki.api_v2.public.utils.get_object import get_page_or_404
from wiki.pages.logic.acl import get_acl_schema
from wiki.pages.logic.etalon import get_etalon_page
from wiki.sync.connect.base_organization import BaseOrganization


def serialize(x) -> PageSchema:
    return PageSchema(id=x[0], slug=x[1])


@di
@legacy_org_ctx
def get_inherited_acl(request, organization: BaseOrganization, idx: int) -> AclSchema:
    try:
        page = get_page_or_404(organization, pk=idx)
    except IsStubPage:
        page = get_etalon_page(organization, idx=idx)

    log_slug(request, slug=page.slug)
    assert_has_access(request.user, organization, page, Action.READ)

    return get_acl_schema(page, organization, force_inheritance=True)
